#----------------------------------------------------------------------------
# Author: An-Chi Ho
# Date: 20th May 2021
#
# This script shows how to load irregular grid data by Start(), then regrid it
# by s2dv::CDORemap in the workflow. It is a solution before Start() can deal
# with irregular regridding directly.
#----------------------------------------------------------------------------

library(startR)
#NOTE: The following library() and source() are not needed when CDORemap() is updated
#      in the installed version.
library(easyNCDF)
library(ncdf4)
library(ClimProjDiags)
source('/home/Earth/aho/aho-testtest/startR/issue/issue_CDORemap_irregular/CDORemap_irregular.R')
source('/home/Earth/aho/startR/R/Utils.R')
source('/home/Earth/aho/s2dverification/R/ArrayToNetCDF.R')

path <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/cmcc-cm2-sr5/cmip6-dcppA-hindcast_i1p1/',
               'DCPP/CMCC/CMCC-CM2-SR5/dcppA-hindcast/$member$/Omon/$var$/gn/v20210312/',
               '$var$_*_s$sdate$-$member$_gn_$aux$.nc')

data <- Start(dataset = path,
              var = 'tos',
              sdate = c('1960', '1961'),
              aux = 'all',
              aux_depends = 'sdate',
              j = 'all', 
              i = 'all', 
              time = indices(1:12),
              member = 'r1i1p1f1',
              return_vars = list(j = NULL, i = NULL, 
                                 latitude = NULL, longitude = NULL),
              retrieve = F)

func_regrid <- function(data) {
  lons <- attr(data, 'Variables')$common$longitude
  lats <- attr(data, 'Variables')$common$latitude
  data <- CDORemap_irregular(data, lons, lats, grid = 'r360x180', 
                             method = 'bil', crop = FALSE)
  lons_reg <- data[['lons']]
  lats_reg <- data[['lats']]
  return(list(data = data[[1]], lats = lats_reg, lons = lons_reg))
}

#NOTE: target_dims can only have 'j' and 'i'. It should be fixed when CDORemap is fixed
#      and installed.
step <- Step(fun = func_regrid,
             target_dims = list(data = c('var', 'aux', 'j', 'i')),
             output_dims = list(data = c('var', 'aux', 'lon', 'lat'), 
                                lats = 'lat', lons = 'lon'),
             use_attributes = list(data = "Variables"))
wf <- AddStep(data, step)

res <- Compute(workflow = wf$data,
               chunks = list(sdate = 2, time = 2))

names(res)
#[1] "data" "lats" "lons"
dim(res$data)
#    var     aux     lon     lat dataset   sdate    time  member 
#      1       1     360     180       1       2      12       1 
dim(res$lon)
#    lon dataset   sdate    time  member 
#    360       1       2      12       1 
dim(res$lat)
#    lat dataset   sdate    time  member 
#    180       1       2      12       1 

s2dv::PlotEquiMap(t(res$data[1, 1, , , 1, 1, 1, 1]),
                  lon = res$lons[, 1, 1, 1, 1],
                  lat = res$lats[, 1, 1, 1, 1])


